package venice;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.net.URL;


/**
 * This class instantiates AppModule and loads the default
 * model, if one is specified using "initialmodelurl" parameter
 * in the HTML document launching this Applet. The parameter
 * must be a valid URL. If the file name ends with ".venice",
 * then the file is assumed to be a Venice package and is
 * loaded accordingly.
 */
public class Applet extends javax.swing.JApplet
{
    /**
     * Default constructor. Creates a new AppModule instance
     * and tells it we are an Applet.
     */ 
    public Applet()
    {
	mAppModule = new AppModule(true);
    }
    
    /**
     * Adds some dummy content to the Applet window. The
     * Applet window most probably is embedded in a
     * browser window. Venice uses its own window, so it
     * would look quite silly if the embedded Applet window
     * is empty.
     */
    public void init()
    {
	JLabel label = new JLabel("VENICE!");
	
	label.setHorizontalAlignment(JLabel.CENTER);
	
	//Add border.  Should use createLineBorder, but then the bottom
	//and left lines don't appear -- seems to be an off-by-one error.
	label.setBorder(BorderFactory.createMatteBorder(1,1,2,2,Color.black));
	
	getContentPane().add(label, BorderLayout.CENTER);
	
	String myURLString = getParameter(INITIAL_MODEL_URL_STRING);
	
	// FIXME: command should be run with swingutilities invoke later
	// If an initial model URL was supplied, load it.
	if(myURLString != null) {
	    if(myURLString.endsWith(".venice"))
		(new LoadURLCommand(myURLString)).execute();
	    else
		(new ImportGXLURLCommand(myURLString)).execute();
	}
    }
    
    /**
     * Returns information about parameters understood by this applet.
     * This applet understands the parameter "initialmodelurl".
     * It is the URL to load when the applet is launched.
     * <p>Remember: URL must be at the same server as the applet and
     * sometimes it is not so nice to have an autoloading model.
     *
     * @return list of parameters understood by this applet
     */
    public String[][] getParameterInfo()
    {
	String[][]	myInfo = {
	    {INITIAL_MODEL_URL_STRING,   "url",     "location of initial model"}
	};
	
	return myInfo;
    }
    
    /** Reference to the AppModule instance created by this Applet. */
    AppModule mAppModule = null;
    
    static final String	INITIAL_MODEL_URL_STRING = "initialmodelurl";
}







