package venice;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.*;


/**
 * Application class is the class that should be instantiated
 * when Venice is to be used in application form. It creates
 * an AppModule instance and loads a model, if one was specified
 * from the command line.
 * 
 */
public class Application
{
    /**
     * This method creates a new instance of
     * <code>AppModule</code> not in Applet mode.
     * 
     * <p>It can be given a parameter at startup.
     * The parameter is the name of the model to load at startup.
     * If the file name begins with a "http://" it is assumed to be
     * an URL. If the file name ends with a ".venice" the file is
     * assumed to be a Venice package.
     */
    public static void main(String args[])
    {
	if(args.length > 0) {
	    for(int i = 0; i < args.length; ++i) {
		if(args[i].equals("--help") || args[i].equals("-h")) {
		    System.out.println("Usage: java -jar venice.jar [FILENAME]\n");
		    System.out.println("        FILENAME is the GXL model to load initially.");
		    System.out.println("        If the filename starts with 'http://' it is");
		    System.out.println("        recognized as an URL and the document is fetched");
		    System.out.println("        with HTTP");
		    return;
		}
	    }
	}
	
	AppModule myAppModule = new AppModule(false);
	
	Visualization myVisualization = 
	    myAppModule.getGui().getVisualization();
	ModelStorage myModelStorage = 
	    myAppModule.getModelStorage();
	
	// FIXME: commands should be handled with SwingUtilities.invokeLater
	if(args.length > 0) {
	    if(args[0].startsWith("http://")) {
		// The parameter is an URL, now lets see if it is a Venice package
		if(args[0].endsWith(".venice"))
		    Util.invokeCommandLater(new LoadURLCommand(args[0]));
		else
		    Util.invokeCommandLater(new ImportGXLURLCommand(args[0]));
	    } else {
		// The parameter is a file.
		if(args[0].endsWith(".venice"))
		    Util.invokeCommandLater(new LoadFileCommand(args[0]));
		else
		    Util.invokeCommandLater(new ImportGXLFileCommand(args[0]));
	    }
	}
    }
}
