package venice;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;

/**
 * Abstract superclass for different GXL model importing classes
 *
 * Performs loading a GXL model file from an inputstream which 
 * is returned by getInputStream()

 * If the user commits to this action then the
 * request is forwarded to the current ModelStorage
 * and CreateVisualizationTreeCommand.
 * 
 * @author Petteri Kamppuri
 * @author Hannu Laurila
 */
public abstract class ImportGXLCommand extends Command
{
    /**
     * Prompts user for a GXL file location and opens it for reading.
     * Returns null if operation is canceled or open fails.
     */
    protected abstract InputStream getInputStream();

    /**
     * Loads the GXL to model storage and then issues
     * <code>CreateVisualizationTreeCommand</code> to
     * create the visual representation of the model.
     */
    public void execute()
    {
        InputStream myGXLIs = getInputStream();        

        // check if operation was canceled
        if (myGXLIs == null)
            return;
	try {
	    ModelStorage myModel = getAppModule().getModelStorage();
	    myModel.clear();
	    myModel.loadGXL(myGXLIs);
	} catch (ParserConfigurationException pce) {
	    Util.handleException(pce);
	} catch (SAXException se) {
	    Util.handleException(se);
	} catch (IOException ioe) {
	    Util.handleException(ioe);
	}
        
        Command myCreateVisTreeCmd = new CreateVisualizationTreeCommand();
        
        myCreateVisTreeCmd.execute();
    }
}
