package venice;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * Performs loading a GXL model file from a file.
 * The user is displayed a dialog in which he can
 * select which file to load.
 * 
 * @author Petteri Kamppuri
 * @author Hannu Laurila
 */
public class ImportGXLFileCommand extends ImportGXLCommand
{
    private String mSuppliedFilename;

    /**
     * constructs a command which tries loading from supplied file.
     */
    public ImportGXLFileCommand(String inFilename)
    {
	mSuppliedFilename = inFilename;
    }

    /**
     * constructs a command which asks user for file name.
     */
    public ImportGXLFileCommand()
    {
	this(null);
    }

    /**
     * Displays a dialog so the user can choose which
     * file to load. Opens file for reading and returns the stream.
     */
    protected InputStream getInputStream()
    {
	File myFile;

	if (mSuppliedFilename == null) {
	    final JFileChooser fc = new JFileChooser();
	    fc.addChoosableFileFilter(new GXLFileFilter());
	    int returnVal = fc.showOpenDialog(null);
        
	    if (returnVal != JFileChooser.APPROVE_OPTION)
		return null;
	    
	    myFile = fc.getSelectedFile();
	} else {
	    myFile = new File(mSuppliedFilename);
	}
        
        try {
            InputStream myIs = new FileInputStream(myFile);
            return myIs;
        }
        catch (IOException myEx) {
            Util.handleException(myEx);
        }
        
        return null;
    }
}
