package venice;

import java.io.*;
import javax.swing.*;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;

/**
 * LoadCommand implements loading of .venice-files which contains both
 * the GXL model and the visualization layout.
 *
 * LoadCommand is an abstract class which can load the file from any
 * InputStream. Subclasses of this command must implement getInputStream()
 * method which opens input stream for needing.
 *
 * @author Hannu Laurila
 */
public abstract class LoadCommand extends Command
{
    /**
     * returns the input stream where to load data from or null
     * to cancel load operation. Likely consults user to find the stream.
     */
    protected abstract InputStream getInputStream();

    public void execute()
    {
        Visualization myVisualization = 
            getAppModule().getGui().getVisualization();
        ModelStorage myModelStorage = 
            getAppModule().getModelStorage();  

        InputStream myIs = getInputStream();

        if (myIs == null)
            return;
    
        try {    
            VeniceFileHandler myVfh = new VeniceFileHandler();
            
            myVfh.readFrom(myIs);

            InputStream myVisIs = myVfh.getVisualizationInputStream();
            myVisualization.clear();
            myVisualization.loadLayout(myVisIs);

            InputStream myGXLIs = myVfh.getGXLInputStream();
            myModelStorage.clear();
            myModelStorage.loadGXL(myGXLIs);

        }
		catch (ParserConfigurationException pce) {
			Util.handleException(pce);
		}
		catch (SAXException se) {
			Util.handleException(se);
		}
        catch (IOException ex) {
            Util.handleException(ex);
        }
    }
}
