package venice;

import java.net.*;
import java.io.*;
import javax.swing.*;

/**
 * Implements .venice-loading from URL.
 *
 * @author Hannu Laurila
 */
public class LoadURLCommand extends LoadCommand
{
    private String mSuppliedURL;

    /**
     * constructs an LoadURLCommand which tries loading from supplied URL
     */
    public LoadURLCommand(String inURL)
    {
	mSuppliedURL = inURL;
    }
    
    /**
     * constructs an LoadURLCommand which prompts user for URL
     */
    public LoadURLCommand()
    {
	this(null);
    }

    /**
     * asks user for the URL of the .venice files and opens connection
     * to the URL.
     */
    protected InputStream getInputStream()
    {
        String myUrlString;

	if (mSuppliedURL == null) {
	    myUrlString = 
		JOptionPane.showInputDialog(null, 
					    "URL of VENICE Visualization file:",
					    "Open",
					    JOptionPane.QUESTION_MESSAGE);
	} else {
	    myUrlString = mSuppliedURL;
	}
        
        if (myUrlString == null)
            return null;

        try {
            URL myUrl = new URL(myUrlString);
            InputStream myIs = myUrl.openStream();
            return myIs;
        }
        catch (IOException ex) {
            Util.handleException(ex);
        }

        return null;
    }
}
