/**
 * @(#) MenuToolBar.java
 */
package venice;


import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
import javax.swing.*;

/**
 * MenuToolBar is implementation of menubar and toolbar used in Venice user interface.
 * <p>
 * This class provides all needed Menu and ToolBar elements.
 * 
 * @author Antti Pietarinen
 * @author Petteri Kamppuri
 */

public class MenuToolBar
{
    /** Menu mMenuBar */
	JMenuBar			mMenuBar			= new JMenuBar();
	
    /** File menu*/
/*
	JMenu				mFileMenu			= new JMenu("File");
	JMenuItem			mSaveMenuItem			= new JMenuItem("Save model");
	JMenuItem			mLoadMenuItem			= new JMenuItem("Load model from file");
        JMenuItem			mLoadHTTPMenuItem		= new JMenuItem("Load model from URL");
    JMenuItem                       mLoadLayoutMenuItem                 = new JMenuItem("Load layout from file");
    JMenuItem                       mLoadLayoutURLMenuItem              = new JMenuItem("Load layout from URL");
    JMenuItem                       mSaveLayoutMenuItem                 = new JMenuItem("Save layout to file");
	JMenuItem			mExitMenuItem			= new JMenuItem("Exit");	
*/
    JMenu                       mFileMenu			= new JMenu("File");
    JMenuItem                   mLoadFileMenuItem               = new JMenuItem("Open from file...");
    JMenuItem                   mLoadURLMenuItem                = new JMenuItem("Open from URL...");
    JMenuItem                   mSaveFileMenuItem               = new JMenuItem("Save to file...");
    JMenuItem			mImportGXLURLMenuItem		= new JMenuItem("Import GXL from URL...");
    JMenuItem			mImportGXLFileMenuItem		= new JMenuItem("Import GXL from file...");
    JMenuItem			mExportGXLFileMenuItem		= new JMenuItem("Export GXL to file...");
    JMenuItem                   mPrintMenuItem                  = new JMenuItem("Print current view...");
    JMenuItem			mExitMenuItem			= new JMenuItem("Exit");

    /** Edit menu*/
	JMenu				mEditMenu			= new JMenu("Edit");
	JMenuItem			mGroupMenuItem			= new JMenuItem("Group");
	JMenuItem			mUngroupMenuItem		= new JMenuItem("Ungroup");
    /** View menu*/
	JMenu				mViewMenu			= new JMenu("View");
	JMenuItem			mDependenciesDownMenuItem	= new JMenuItem("Relationships down");
	JMenuItem			mDependenciesUpMenuItem		= new JMenuItem("Relationships up");
	JMenuItem			mShowInformationMenuItem	= new JMenuItem("Show information");
	// Package status submenu
	JMenu				mPackageStatusMenu		= new JMenu("Package");
	JMenuItem 		mOpenMenuItem			= new JMenuItem("Open");
	JMenuItem 		mClosedMenuItem			= new JMenuItem("Close");
	JMenuItem 		mTransparentMenuItem		= new JMenuItem("Transparent");
	// Interfaces submenu
	JMenu				mInterfacesMenu			= new JMenu("Interfaces");	
	JRadioButtonMenuItem		mHideInterfacesMenuItem		= new JRadioButtonMenuItem("Hide all");
        JMenuItem	 	        mHideSelectedMenuItem		= new JMenuItem("Hide selected");
	JCheckBoxMenuItem			mShowInterfaceContentMenuItem 	= new JCheckBoxMenuItem("Show content");
	JCheckBoxMenuItem			mShowOfferedInterfacesMenuItem 	= new JCheckBoxMenuItem("Show offered");
	JCheckBoxMenuItem			mShowRequiredInterfacesMenuItem = new JCheckBoxMenuItem("Show required");
	// Show labels submenu
	JMenu				mLabelsMenu			= new JMenu("Show labels");	
    	// FIXME! The state of these checkboxes are not restored when a .venice file is loaded.
    	// That can cause UI to be in the wrong state.
	JCheckBoxMenuItem		mDependenciesMenuItem		= new JCheckBoxMenuItem("Relationships",false);
	JCheckBoxMenuItem		mInterfacesLabelMenuItem	= new JCheckBoxMenuItem("Interfaces",true);
	JCheckBoxMenuItem		mPackagesLabelMenuItem		= new JCheckBoxMenuItem("Packages",true);
	JCheckBoxMenuItem		mComponentsLabelMenuItem	= new JCheckBoxMenuItem("Components",true);
	// Colors submenu
	JMenu				mColorsSubmenu			= new JMenu("Colors");	
	JRadioButtonMenuItem		mBlackAndWhiteMenuItem		= new JRadioButtonMenuItem("Black and white", true);
	JRadioButtonMenuItem 		mColorMenuItem			= new JRadioButtonMenuItem("Color",false);
    /** Zoom menu*/
	JMenu				mZoomMenu			= new JMenu("Zoom");
	JMenuItem			mFitToScreenMenuItem		= new JMenuItem("Fit to screen");
	JMenuItem			mZoomToSelectionMenuItem	= new JMenuItem("Zoom to selection");
    /** Selection menu*/
	JMenu				mSelectionMenu			= new JMenu("Selection");
	JMenuItem			mSelectAllMenuItem		= new JMenuItem("Select all");
	JMenuItem			mSelectRelatedMenuItem		= new JMenuItem("Select related");
	JMenuItem			mSelectUnrelatedMenuItem	= new JMenuItem("Select unrelated");
	JMenuItem			mSelectByTypeMenuItem		= new JMenuItem("Select by type");
	JMenuItem			mSelectByNameMenuItem		= new JMenuItem("Select by name");
	JMenuItem			mInvertSelectionMenuItem	= new JMenuItem("Invert selection");
	JMenuItem			mUnselectMenuItem		= new JMenuItem("Unselect");
    /** Filter menu */
	JMenu				mFilterMenu			= new JMenu("Filtering");
	JMenuItem			mFilterByNameMenuItem		= new JMenuItem("Filter by name");
	JMenuItem			mFilterByTypeMenuItem		= new JMenuItem("Filter by type");
	JMenuItem			mFilterSelectedMenuItem		= new JMenuItem("Filter selected");
	JMenuItem			mFilterUnselectedMenuItem	= new JMenuItem("Filter unselected");
	JMenuItem			mFilterRelatedMenuItem		= new JMenuItem("Filter related");
	JMenuItem			mFilterUnrelatedMenuItem	= new JMenuItem("Filter unrelated");
	JMenuItem			mShowEverythingMenuItem		= new JMenuItem("Show everything");
    /** Help menu*/
	JMenu				mHelpMenu			= new JMenu("Help");
        JMenuItem                       mLicenseMenuItem                = new JMenuItem("Licensing information...");
	JMenuItem			mAboutMenuItem			= new JMenuItem("About VENICE...");
	
	// Debug menu
	JMenu				mDebugMenu			= new JMenu("Debug");

    /** Tool bar*/
	JToolBar			mToolBar			= new JToolBar();

	JComboBox box = null;	
	JTextField tField = new JTextField();
	JPopupMenu bMenu = null;
	JMenuItem bMenuFilterByName = null;
	JMenuItem bMenuFilterByType = null;
	JMenuItem bMenuSelectByName = null;
	JMenuItem bMenuSelectByType = null;

	JLabel label = null;
	
	JToggleButton pan = null;
	JToggleButton select = null;
	JToggleButton zoom = null;	
	
	JButton zoomin = null;
	JButton zoomout = null;
	
	JPanel napit = null;
	JPanel napit0 = null;
	JPanel mFinalPanel = null;
	JPopupMenu zMenu = null;
	JSlider zoomVol = null;

    /**
     * Constructor which sets up menubar and toolbar.
     * @param inIsApplet boolean - Information is user applet or application. True is applet, false application.
     * @param inFrame JFrame - JFrame that users this class.
     */
	public MenuToolBar(boolean inIsApplet, JFrame inFrame)
	{
		setupMenuBar(inIsApplet);
		setupToolBar(inFrame);
		initMenuToolBarHandlers(inIsApplet);
		setDefaultMenuBar();
		inFrame.setJMenuBar(mMenuBar);
		
		inFrame.getContentPane().add(mToolBar, BorderLayout.NORTH);

		inFrame.validate();
	}
    /**
     * Method that returns JMenuBar element constructed in this class.
     * @return JMenuBar
     */
	public JMenuBar getJMenuBar()
	{
		return mMenuBar;
	}
    /**
     * Method that returns JToolBar element constructed in this class.
     * @return JTooluBar
     */
	public JToolBar getJToolBar()
	{
		return mToolBar;
	}
    /**
     * Method that returns Debug JMenu element constructed in this class.
     * @return JMenu
     */
	public JMenu getDebugMenu()
	{
		return mDebugMenu;
	}	
    /**
     * Method that sets JMenuBar elements in case of VPackage(s) selected.
     * @param state 
     */
	public void setVPackageMenuBar(boolean hasOffered, boolean hasRequired, boolean offShow, boolean reqShow)
	{
		mGroupMenuItem.setEnabled(true);
		mUngroupMenuItem.setEnabled(true);
		
		mDependenciesDownMenuItem.setEnabled(true);
		mDependenciesUpMenuItem.setEnabled(true);
		mShowInformationMenuItem.setEnabled(false);
		mPackageStatusMenu.setEnabled(true);
		mZoomToSelectionMenuItem.setEnabled(true);
		
		mSelectRelatedMenuItem.setEnabled(true);
		mSelectUnrelatedMenuItem.setEnabled(true);

		mFilterSelectedMenuItem.setEnabled(true);
		mFilterRelatedMenuItem.setEnabled(true);
		mFilterUnrelatedMenuItem.setEnabled(true);

		mHideSelectedMenuItem.setEnabled(false);
		mShowInterfaceContentMenuItem.setEnabled(false); 

		mShowOfferedInterfacesMenuItem.setEnabled(hasOffered);
		mShowOfferedInterfacesMenuItem.setSelected(offShow);
		mShowRequiredInterfacesMenuItem.setEnabled(hasRequired);
		mShowRequiredInterfacesMenuItem.setSelected(reqShow);
	}
    /**
     * Method that sets JMenuBar elements in case of VComponent(s) selected.
     */
	public void setVComponentMenuBar(boolean hasOffered, boolean hasRequired,boolean offShow, boolean reqShow)
	{
		mGroupMenuItem.setEnabled(false);  
		mUngroupMenuItem.setEnabled(false);
		
		mDependenciesDownMenuItem.setEnabled(true);
		mDependenciesUpMenuItem.setEnabled(true);
		mShowInformationMenuItem.setEnabled(false);
		mPackageStatusMenu.setEnabled(false);
		mZoomToSelectionMenuItem.setEnabled(true);
		
		mSelectRelatedMenuItem.setEnabled(true);
		mSelectUnrelatedMenuItem.setEnabled(true);
		
		mFilterSelectedMenuItem.setEnabled(true);
		mFilterRelatedMenuItem.setEnabled(true);
		mFilterUnrelatedMenuItem.setEnabled(true);

		mHideSelectedMenuItem.setEnabled(false);
		mShowInterfaceContentMenuItem.setEnabled(false);

		mShowOfferedInterfacesMenuItem.setEnabled(hasOffered);
		mShowOfferedInterfacesMenuItem.setSelected(offShow);
		mShowRequiredInterfacesMenuItem.setEnabled(hasRequired);
		mShowRequiredInterfacesMenuItem.setSelected(reqShow);                  
	}
    /**
     * Method that sets JMenuBar elements in case of VInterface(s) selected.
     */
	public void setVInterfaceMenuBar(boolean contentShown)
	{
		mGroupMenuItem.setEnabled(false);
		mUngroupMenuItem.setEnabled(false);
		
		mDependenciesDownMenuItem.setEnabled(false);
		mDependenciesUpMenuItem.setEnabled(false);
		mShowInformationMenuItem.setEnabled(false);
		mPackageStatusMenu.setEnabled(false);
		mZoomToSelectionMenuItem.setEnabled(true);
		
		mSelectRelatedMenuItem.setEnabled(true);
		mSelectUnrelatedMenuItem.setEnabled(true); 
		
		mFilterSelectedMenuItem.setEnabled(true); 
		mFilterRelatedMenuItem.setEnabled(true);
		mFilterUnrelatedMenuItem.setEnabled(true);        

		mHideSelectedMenuItem.setEnabled(true);
		mShowInterfaceContentMenuItem.setEnabled(true);
		mShowInterfaceContentMenuItem.setSelected(contentShown);

		mShowOfferedInterfacesMenuItem.setEnabled(false);   
		mShowRequiredInterfacesMenuItem.setEnabled(false);
	}
    /**
     * Method that sets JMenuBar elements in case of VDependencies selected.
     */
	public void setDependencyMenuBar()
	{
		setDefaultMenuBar();
		mShowInformationMenuItem.setEnabled(false);
		mSelectRelatedMenuItem.setEnabled(true);
		mSelectUnrelatedMenuItem.setEnabled(true); 
	}
    /**
     * Method that sets default JMenuBar.
     */
	public void setDefaultMenuBar()
	{
		/*group*/
		mGroupMenuItem.setEnabled(false);
		mUngroupMenuItem.setEnabled(false); 
		/*view*/
		mDependenciesDownMenuItem.setEnabled(false);
		mDependenciesUpMenuItem.setEnabled(false);
		mShowInformationMenuItem.setEnabled(false);
		mPackageStatusMenu.setEnabled(false);

		mHideSelectedMenuItem.setEnabled(false);
		mShowInterfaceContentMenuItem.setEnabled(false);
		mShowOfferedInterfacesMenuItem.setEnabled(false);   
		mShowRequiredInterfacesMenuItem.setEnabled(false);
		
		/*zoom*/
		mZoomToSelectionMenuItem.setEnabled(true);
		
		/*select*/
		mSelectRelatedMenuItem.setEnabled(false);
		mSelectUnrelatedMenuItem.setEnabled(false);
		
		/*filter*/
		mFilterSelectedMenuItem.setEnabled(false);
		mFilterRelatedMenuItem.setEnabled(false);
		mFilterUnrelatedMenuItem.setEnabled(false);
	}
	protected void setupMenuBar(boolean inIsApplet)
	{
	    // File menu

            mFileMenu.add(mLoadURLMenuItem);

            if (!inIsApplet) {
                mFileMenu.add(mLoadFileMenuItem);
                mFileMenu.add(mSaveFileMenuItem);
            }

            mFileMenu.addSeparator();
                
	    mFileMenu.add(mImportGXLURLMenuItem);

	    if(!inIsApplet) {
                mFileMenu.add(mImportGXLFileMenuItem);
                mFileMenu.add(mExportGXLFileMenuItem);
            }

	    if(!inIsApplet) {
                mFileMenu.addSeparator();
                mFileMenu.add(mPrintMenuItem);
                mFileMenu.addSeparator();
                mFileMenu.add(mExitMenuItem);
            }

	    mMenuBar.add(mFileMenu);

	    // Edit menu
	    
	    mEditMenu.add(mGroupMenuItem);
	    mEditMenu.add(mUngroupMenuItem); 
	    mMenuBar.add(mEditMenu);
	    mEditMenu.setEnabled(false);
	    // View menu
		
	    mViewMenu.add(mDependenciesDownMenuItem);
	    mViewMenu.add(mDependenciesUpMenuItem);
	    mViewMenu.add(mShowInformationMenuItem);
	    mShowInformationMenuItem.setEnabled(false);
	    mViewMenu.addSeparator();
	    
	    // Package status submenu
	    
	    mPackageStatusMenu.add(mOpenMenuItem);
	    mPackageStatusMenu.add(mClosedMenuItem);
	    mPackageStatusMenu.add(mTransparentMenuItem);
	    
	    mViewMenu.add(mPackageStatusMenu);
	    
	    //	    mInterfacesMenu.add(mHideInterfacesMenuItem);
	    //	    mInterfacesMenu.add(mHideSelectedMenuItem);
	    //	    mInterfacesMenu.addSeparator();
	    mInterfacesMenu.add(mShowInterfaceContentMenuItem);
	    mInterfacesMenu.addSeparator();
	    mInterfacesMenu.add(mShowOfferedInterfacesMenuItem);
	    mInterfacesMenu.add(mShowRequiredInterfacesMenuItem);
		
	    mViewMenu.add(mInterfacesMenu);
	    
	    // Show labels submenu
	    mLabelsMenu.add(mDependenciesMenuItem);
	    mLabelsMenu.add(mInterfacesLabelMenuItem);
	    mInterfacesLabelMenuItem.setEnabled(false);
	    mLabelsMenu.add(mPackagesLabelMenuItem);
	    mPackagesLabelMenuItem.setEnabled(false);
	    mLabelsMenu.add(mComponentsLabelMenuItem);
	    mComponentsLabelMenuItem.setEnabled(false);
	    
	    mViewMenu.add(mLabelsMenu);
	    
	    mViewMenu.addSeparator();
	    
	    // Colors
	    ButtonGroup myColorsButtonGroup = new ButtonGroup();
	    myColorsButtonGroup.add(mColorMenuItem);
	    myColorsButtonGroup.add(mBlackAndWhiteMenuItem);		
	    
	    mViewMenu.add(mColorMenuItem);
	    mViewMenu.add(mBlackAndWhiteMenuItem);
	    mMenuBar.add(mViewMenu);
	    mColorMenuItem.setEnabled(false);
	    mBlackAndWhiteMenuItem.setEnabled(false);

	    // Zoom menu
	    mZoomMenu.add(mFitToScreenMenuItem);
	    mZoomMenu.add(mZoomToSelectionMenuItem);
	    mMenuBar.add(mZoomMenu);

	    // Selection menu
	    mSelectionMenu.setEnabled(false);
	    mSelectionMenu.add(mSelectRelatedMenuItem);
	    mSelectionMenu.add(mSelectUnrelatedMenuItem);
	    mSelectionMenu.addSeparator();
	    mSelectionMenu.add(mSelectByTypeMenuItem);
	    mSelectionMenu.add(mSelectByNameMenuItem);
	    mSelectionMenu.addSeparator();
	    mSelectionMenu.add(mInvertSelectionMenuItem);
	    mSelectionMenu.add(mUnselectMenuItem);

	    mMenuBar.add(mSelectionMenu);
		
	    // Filtering menu
	    mFilterMenu.setEnabled(false);
	    mFilterMenu.add(mShowEverythingMenuItem);
	    mFilterMenu.addSeparator();
	    mFilterMenu.add(mFilterByNameMenuItem);
	    mFilterMenu.add(mFilterByTypeMenuItem);
	    mFilterMenu.addSeparator();
	    mFilterMenu.add(mFilterSelectedMenuItem);
	    mFilterMenu.add(mFilterUnselectedMenuItem);
	    mFilterMenu.addSeparator();
	    mFilterMenu.add(mFilterRelatedMenuItem);
	    mFilterMenu.add(mFilterUnrelatedMenuItem);
	    
	    mMenuBar.add(mFilterMenu);
	    
	    // Help menu

	    mHelpMenu.add(mLicenseMenuItem);
	    mHelpMenu.addSeparator();
	    mHelpMenu.add(mAboutMenuItem);
	    mMenuBar.add(mHelpMenu);

	    // Debug menu

	    mMenuBar.add(mDebugMenu);		
	}
    
	void setupToolBar(JFrame inFrame)
	{
		URL resource;
		
		tField.setEditable(false);
		tField.setPreferredSize(new Dimension(inFrame.getWidth()-200, 26));
		tField.setMaximumSize(new Dimension(600, 26));
		tField.setMinimumSize(new Dimension(10, 26));
		tField.setEnabled(false);

		bMenu = new JPopupMenu();
		bMenuFilterByName = new JMenuItem("Filter by name");
		bMenuFilterByType = new JMenuItem("Filter by type");
		bMenuSelectByName = new JMenuItem("Select by name");
		bMenuSelectByType = new JMenuItem("Select by type");
		
		bMenu.add(bMenuFilterByName);
		bMenu.add(bMenuFilterByType);
		bMenu.addSeparator();
		bMenu.add(bMenuSelectByName);
		bMenu.add(bMenuSelectByType);
		
		mToolBar.setFloatable(false);
		
		label = new JLabel("Filter / Select:");
		
		// Select button
		resource = this.getClass().getClassLoader().getResource("venice/hand.gif");
		pan = new JToggleButton(new ImageIcon(resource), false);
		
		pan.setToolTipText("Pan");
		pan.setText(null);
		pan.setPreferredSize(new Dimension(30, 26));
		pan.setMaximumSize(new Dimension(30, 26));
		
		resource = this.getClass().getClassLoader().getResource("venice/crosshair.gif");
		select = new JToggleButton(new ImageIcon(resource));
		
		select.setToolTipText("Select");
		select.setText(null);
		select.setPreferredSize(new Dimension(30, 26));
		select.setMaximumSize(new Dimension(30, 26));
		select.setSelected(true);
		
		// Zoom button
		resource = this.getClass().getClassLoader().getResource("venice/zoomin.gif");
		zoom = new JToggleButton(new ImageIcon(resource));
		
		zoom.setToolTipText("Zoom");
		zoom.setText(null);
		zoom.setPreferredSize(new Dimension(30, 26));
		zoom.setMaximumSize(new Dimension(30, 26));
		
		zoomVol = new JSlider(0,100,50);
		zoomVol.setToolTipText("Set zooming volume");
		zoomVol.setPaintLabels(true);
		zoomVol.setPaintTicks(true);
		zoomVol.setMajorTickSpacing(25);
		zoomVol.setMinorTickSpacing(5);
		zoomVol.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
		
		JLabel zLabel = new JLabel("Zoom volume");
		zLabel.setAlignmentX(Component.CENTER_ALIGNMENT); 
		zMenu = new JPopupMenu("Zoom volume");
		zMenu.add(zLabel);
		zMenu.addSeparator();
		zMenu.add(zoomVol);
		
		ButtonGroup myToolBarButtonGroup = new ButtonGroup();
		myToolBarButtonGroup.add(select);
		myToolBarButtonGroup.add(pan);
		myToolBarButtonGroup.add(zoom);
		
		napit0 = new JPanel(new GridLayout(1,3));
		napit0.add(select);
		napit0.add(pan);
		napit0.add(zoom);
		napit = new JPanel(new FlowLayout());
		napit.add(napit0);	
		napit.add(label);
		napit.add(tField);
		mFinalPanel = new JPanel(new BorderLayout());
		mFinalPanel.add(napit,BorderLayout.WEST);	
		
		mToolBar.add(mFinalPanel);
	}

	private void initMenuToolBarHandlers(boolean inIsApplet)
	{
            /* FILE Menubar eventhandlers */

            // Load from URL
            mLoadURLMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new LoadURLCommand().execute();
                    }
                });

            // Load from file
            mLoadFileMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new LoadFileCommand().execute();
                    }
                });
            
            // Save to file
            mSaveFileMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new SaveFileCommand().execute();
                    }
                });

            // Import GXL from URL
            mImportGXLURLMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new ImportGXLURLCommand().execute();
                    }
                });

            // Import GXL from file
            mImportGXLFileMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new ImportGXLFileCommand().execute();
                    }
                });

            // Export GXL to file
            mExportGXLFileMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new ExportGXLFileCommand().execute();
		    }
		});

            // Print
            mPrintMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new PrintCurrentViewCommand().execute();
                    }
                });
            
            // Exit 
            mExitMenuItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        System.exit(0);
                    }
                });

	/* About handler*/
	mAboutMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
                    Util.showAboutDialog(null);
		}
	    });

	/* License handler */
	mLicenseMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
                    Util.showLicenseDialog(null);
		}
	    });
	
	/*Toolbar eventhandlers*/
	// Filter field handler
	tField.addMouseListener (
				 new MouseAdapter() {
					 public void mousePressed( MouseEvent e )
					 { checkForTriggerEvent( e ); } 
					 public void mouseReleased( MouseEvent e )
					 { checkForTriggerEvent( e ); } 
					 private void checkForTriggerEvent( MouseEvent e )
					 {	
					     if ( e.isPopupTrigger() ) 
						 bMenu.show( e.getComponent(),e.getX(), e.getY() );               
					 }
				     }
				 );
	// zoom button right mousebutton handler
	zoom.addMouseListener (
			       new MouseAdapter() {
				       public void mousePressed( MouseEvent e )
				       { checkForTriggerEvent( e ); } 
				       public void mouseReleased( MouseEvent e )
				       { checkForTriggerEvent( e ); } 
				       private void checkForTriggerEvent( MouseEvent e )
				       {	
					   if ( e.isPopupTrigger() ) 
					       zMenu.show( e.getComponent(),e.getX(), e.getY() );               
				       }
				   }
			       );
      	/*Edit eventhandlers*/
	mGroupMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Group pressed");	
		}
	    }
					 );
	mUngroupMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Ungroup pressed");	
		}
	    }
					   );
	/*View eventhandlers*/
	mDependenciesUpMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    //JOptionPane.showMessageDialog(null, "DepUp pressed");
		    (new MoveRelationshipsUpCommand()).execute();
		}
	    }
						  );
	mDependenciesDownMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    //JOptionPane.showMessageDialog(null, "DepDown pressed");	
		    (new MoveRelationshipsDownCommand()).execute();
		}
	    }
						    );
	mShowInformationMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Show info pressed");	
		}
	    }
						   );
	mOpenMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    //JOptionPane.showMessageDialog(null, "Open selected");
			(new OpenPackageCommand()).execute();
		}
	    }
					);      		
	mClosedMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    //JOptionPane.showMessageDialog(null, "Closed selected");	
			(new ClosePackageCommand()).execute();
		}
	    }
					  );
	mTransparentMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    //JOptionPane.showMessageDialog(null, "Transparent selected");	
			(new MakePackageTransparentCommand()).execute();
		}
	    }
					       );
	mHideInterfacesMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    new ShowOfferedInterfacesCommand().execute();

		}
	    }
						  );
	mShowInterfaceContentMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if(!mShowInterfaceContentMenuItem.isEnabled())return;
                    if( !mShowInterfaceContentMenuItem.isSelected() ){
                        new HideInterfaceContentCommand().execute();
                        System.out.println("Making required hide");
                    }
                    else{
                        new ShowInterfaceContentCommand().execute();
                        System.out.println("Making required show");
                    }

		}
	    }
							);
	mShowRequiredInterfacesMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if(!mShowRequiredInterfacesMenuItem.isEnabled())return;
  		    if( !mShowRequiredInterfacesMenuItem.isSelected() ){
                        new HideRequiredInterfacesCommand().execute();
                        System.out.println("Making required hide");
                    }
                    else{
                        new ShowRequiredInterfacesCommand().execute();
                        System.out.println("Making required show");
                    }


		    //new ShowRequiredInterfacesCommand().execute();
		    //JOptionPane.showMessageDialog(null, "Show req. pressed");	
		}
	    }
							  );
	mShowOfferedInterfacesMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if( !mShowOfferedInterfacesMenuItem.isEnabled() )return;
		    if( !mShowOfferedInterfacesMenuItem.isSelected() ){
			new HideOfferedInterfacesCommand().execute();
		    	System.out.println("Making hide");
		    }
		    else{
			new ShowOfferedInterfacesCommand().execute();
			System.out.println("Making show"); 
		    }
		}
	    }
							 );
	mDependenciesMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if(mDependenciesMenuItem.isSelected())
			(new ShowVRelationshipLabelsCommand()).execute();
		    else
			(new HideVRelationshipLabelsCommand()).execute();
		}
	    }
						);
	mInterfacesLabelMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Show int.labels pressed");
		}
	    }
						   );
	mPackagesLabelMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Show pac.labels pressed");
		}
	    }
						 );
	mComponentsLabelMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Show com.labels pressed");
		}
	    }
						   );
	mColorMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Show dep. in colors pressed");	
		}
	    }
					 );
	mBlackAndWhiteMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Show dep. in b&w pressed");	
		}
	    }
						 );
	/*Zoom menu eventhandlers*/
	mFitToScreenMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    new FitToScreenCommand().execute();	
		}
	    }
	);

	mZoomToSelectionMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    new ZoomToSelectionCommand().execute();
		}
	    }
						   );
	/*Selection menu Eventhandlers*/
	mSelectAllMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Select all pressed");	
		}
	    }
					     );
	mSelectRelatedMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Select related pressed");	
		}
	    }
						 );
	mSelectUnrelatedMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Select unrelated pressed");	
		}
	    }
						   );
	mSelectByTypeMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Select by type pressed");	
		}
	    }
						);
	mSelectByNameMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Select by name pressed");	
		}
	    }
						);
	mInvertSelectionMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Invert selection pressed");	
		}
	    }
						   );
	mUnselectMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Unselect pressed");	
		}
	    }
					    );
	/*Filter menu Eventhandlers*/
	mFilterByNameMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Filter by name pressed");	
		}
	    }
						);
	mFilterByTypeMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Filter by type pressed");	
		}
	    }
						);
	mFilterSelectedMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Filter selected pressed");	
		}
	    }
						  );
	mFilterUnselectedMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Filter Unselected pressed");	
		}
	    }
						    );
	mFilterRelatedMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Filter related pressed");	
		}
	    }
						 );
	mFilterUnrelatedMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Filter unrelated pressed");	
		}
	    }
						   );
	mShowEverythingMenuItem.addActionListener(new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    JOptionPane.showMessageDialog(null, "Show everything pressed");	
		}
	    }
						  );
    }
}
