package venice;

import java.util.LinkedList;
import java.util.HashMap;
import java.util.ListIterator;
import java.net.URL;
import java.net.MalformedURLException;

import java.io.FileNotFoundException; 
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.File;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;

/**
 * ModelStorage provides a datastructure and operations for a datastructure containing a GXL-model.
 *
 * @author Sami Ilonen
 *
 */

public class ModelStorage {

	private LinkedList nodes;
	private LinkedList edges;
	private HashMap mElementIndex;
	private HashMap mRelationshipIndex;

	public ModelStorage () {
		nodes = new LinkedList();
		edges = new LinkedList();
		mElementIndex = new HashMap();
		mRelationshipIndex = new HashMap();
	}

	/**
	 * This method is used to load a GXL-model from a given location.
	 *
	 * @param inUrl location where the GXL-file should be read.
	 *
	 */

/* OBSOLETE CODE

	public void loadGXL (String inUrl) {

		URL source = null;
		try {
			source = new URL(inUrl);			
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse(source.openStream(), new GXLReader(this.nodes, this.edges));
			this.createIndex();	
			this.checkRelationships();
		} catch (MalformedURLException mau) {
			System.err.println("ERROR (ModelStorage): Malformed URL!");		
		} catch (IOException ioe) {
			System.err.println("ERROR (ModelStorage): Venice is unable to read from given URL location.\nFile in location \""+inUrl+"\" does not exist or is unaccessible.\n");
		} catch (SAXParseException spe) {
			System.err.println("ERROR (ModelStorage): Missing whitespace before SYSTEM literal URI.\n");
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	/**
	 * This method is used to load a GXL-model from a given java.io.File.
	 *
	 * @param inFile is a java.io.File containing the GXL-file.
	 *
	 */

	/*
	public void loadGXL (File inFile) {

		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse(inFile, new GXLReader(this.nodes, this.edges));
			this.checkRelationships();
			this.createIndex();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	END OF OBSOLETE CODE 
*/

	/**
	 * This method is used to load a GXL-model from a given InputStream. 
	 *  
	 *
	 * @param inStream is a java.io.InputStream containing the GXL-model.
	 *
	 */
	public void loadGXL (InputStream inStream) throws IOException, SAXException, ParserConfigurationException {

		SAXParserFactory myFactory = SAXParserFactory.newInstance();
		SAXParser mySaxParser = myFactory.newSAXParser();

	   	mySaxParser.parse(inStream, new GXLReader(this.nodes, this.edges));
	   	inStream.close();
	   	this.checkRelationships();
	   	this.createIndex();
	   	this.checkRelationships();
	}

/* OBSOLETE CODE
	public void writeGXL (String inFile) {

		ListIterator it;
		MElement myTempElement;
		MRelationship myTempRel;
		it = nodes.listIterator();
		try {
			FileWriter myFileWriter = new FileWriter(inFile);
			myFileWriter.write(this.GXLStart());
			myFileWriter.flush();			
			while (it.hasNext()) {
				myTempElement = (MElement) it.next();
				myFileWriter.write(myTempElement.toXML());
				myFileWriter.flush();				
			}

			it = edges.listIterator();
			while (it.hasNext()) {
				myTempRel = (MRelationship) it.next();
				myFileWriter.write(myTempRel.toXML());
				myFileWriter.flush();
			}
			myFileWriter.write(this.GXLEnd());
			myFileWriter.flush();
		} catch (IOException ioe) {
			System.err.println("ERROR: Can't write to file");	
		}
	}
*/
	/**
	 * This method is used to write the information from the model to a OutputStream
	 *
	 */
	public void writeGXL (OutputStream ioStream) throws IOException {

		ListIterator it;
		MElement myTempElement;
		MRelationship myTempRel;
		it = nodes.listIterator();
		ioStream.write(this.GXLStart().getBytes());	
		ioStream.flush();			
		while (it.hasNext()) {
	   		myTempElement = (MElement) it.next();
	   		ioStream.write(myTempElement.toXML().getBytes());
	   		ioStream.flush();				
	   	}

	   	it = edges.listIterator();
	   	while (it.hasNext()) {
	   		myTempRel = (MRelationship) it.next();
	   		ioStream.write(myTempRel.toXML().getBytes());
	   		ioStream.flush();
	   	}
	   	ioStream.write(this.GXLEnd().getBytes());
	   	ioStream.flush();
	   	ioStream.close();
	}		

	private void checkRelationships() {
		ListIterator it;
		Object temp;
		MRelationship myTempRelationship;
		
		it = edges.listIterator();
		
		while (it.hasNext()) {
			temp = it.next();
			myTempRelationship = (MRelationship) temp;
			if (this.getElement(myTempRelationship.getSource()) == null) {
				it.remove();
			}
			else if (this.getElement(myTempRelationship.getDestination()) == null) {
				it.remove();
			}
		}	
	}

	private String GXLStart() {
		 String outString = "";
		 outString = "<?xml version=\"1.0\"?>\n";
		 outString += "<!-- generated with Venice (http://www.cs.helsinki.fi/group/venice/) -->\n";
		 outString += "<!DOCTYPE gxl SYSTEM \"http://www.cs.helsinki.fi/group/venice/\">\n";
		 outString += "<gxl>\n";
		 outString += "<graph id=\"\" edgeids=\"true\">\n";
		 return outString;
	}

	private String GXLEnd() {
		 String outString = "";

		 outString += "</graph>\n";
		 outString += "</gxl>\n";
		 		
		 return outString;	
	}

	/**
	 * This method is used to clear the existing data from this GXL-model.
	 *
	 */
	public void clear () {
		this.nodes = new LinkedList();
		this.edges = new LinkedList();
		this.mElementIndex = new HashMap();
		this.mRelationshipIndex = new HashMap();
	}

	/**
	 * Returns all relationships in the model
	 *
	 * @return Returns all relatioships in the model.
	 */
	public LinkedList getAllRelationships() {
		return edges;
	}

	/**
	 * Returns the parent MElement of a certain MElement
	 *
	 * @param inId Id of the child which parent is returned.
	 * @return Parent MElement if available, null if no parent exist.
	 *
	 */
	public MElement getParent (String inId) {

		ListIterator itEdge;
		MRelationship temp;

		itEdge = edges.listIterator();

		while (itEdge.hasNext()) {
			temp = (MRelationship) itEdge.next();
			if ( (temp instanceof MContainment) && (temp.getDestination().equals(inId)) ) {
				return this.getElement(temp.getSource());
			}
		}
		return null;
	}

	/**
	 * Returns the parent MElement of a certain MElement
	 *
	 * @param inId Id of the parent which children are returned.
	 * @return Linked list of MElements, null if no children exist.
	 *
	 */
	public LinkedList getChildren (String inId) {

		LinkedList out = new LinkedList();
		ListIterator it;
		MRelationship temp;

		it = edges.listIterator();

		while (it.hasNext()) {
			temp = (MRelationship) it.next();
			if ( (temp instanceof MContainment) && (temp.getSource().equals(inId)) ) {
				out.addLast(this.getElement(temp.getDestination()));
			}
		}
		return out;
	}

	/**
	 * Returns the MElement with inId as id.
	 *
	 * @param inId Id of the MElement which is required.
	 * @return MElement with Id = inId, null if not found.
	 *
	 */
	public MElement getElement (String inId) {

		ListIterator it;
		MElement temp;

		it = nodes.listIterator();
		
		while (it.hasNext()) {
			temp = (MElement) it.next();
			if (temp.getId().equals(inId)) {
				return temp;
			}
		}
		return null;
		
/*
		if (mElementIndex.get(inId) instanceof MElement) {
			return (MElement) mElementIndex.get(inId);
		}
		else {
			return null;
		}
*/		
		
	}

	public MRelationship getRelationship (String inId) {

		ListIterator it;
		MRelationship temp;

		it = edges.listIterator();

		while (it.hasNext()) {
			temp = (MRelationship) it.next();
			if ( (temp instanceof MRelationship) && (temp.getId().equals(inId)) ) {
				return temp;
			}
		}
		return null;
	}			

	/**
	 * Returns the root nodes of the model.
	 *
	 * @return Linked list of MElements which do not have parents.
	 *
	 */
    public LinkedList getRootNodes() {

		LinkedList out = new LinkedList();
		ListIterator it;
		MElement temp;

		it = nodes.listIterator();

		while (it.hasNext()) {
			temp = (MElement) it.next();
			if ( (this.getParent(temp.getId()) == (null)) ) {
				out.addLast(temp);
			}
		}
		return out;
    }


	/**
	 * Returns the dependencies of a certain MElement
	 *
	 * @return Linked list of MRelationships which are dependencies of MElement which id is inID
	 *
	 */
    public LinkedList getDependencies(String inId) {

		LinkedList out = new LinkedList();
		ListIterator it;
		MRelationship temp;

		it = edges.listIterator();

		while (it.hasNext()) {
			temp = (MRelationship) it.next();
			if ( (temp instanceof MDependency) && (temp.getSource().equals(inId)) ) {
				out.addLast(temp);
			}
		}
		return out;
	}

	/**
	 * Returns the realizations of a certain MElement
	 *
	 * @return Linked list of MRelationships which are realizations of MElement which id is inID.
	 *
	 */
	public LinkedList getRealizations(String inId) {

		LinkedList out = new LinkedList();
		ListIterator it;
		MRelationship temp;

		it = edges.listIterator();

		while (it.hasNext()) {
			temp = (MRelationship) it.next();
			if ( (temp instanceof MRealization) && (temp.getSource().equals(inId)) ) {
				out.addLast(temp);
			}
		}
		return out;
	}

	/**
	 * Returns the generalizations of a certain MElement
	 *
	 * @return Linked list of MRelationships which are generalizations of MElement which id is inID
	 *
	 */
	public LinkedList getGeneralizations(String inId) {

		LinkedList out = new LinkedList();
		ListIterator it;
		MRelationship temp;

		it = edges.listIterator();

		while (it.hasNext()) {
			temp = (MRelationship) it.next();
			if ( (temp instanceof MGeneralization) && (temp.getSource().equals(inId)) ) {
				out.addLast(temp);
			}
		}
		return out;
	}

	public void addElementToIndex (MElement inElement) {

		if (mElementIndex.containsKey(inElement.getId()) ) {
			// FIXME: Code for duplicates...
		}
		else {
			mElementIndex.put(inElement.getId(), inElement);
		}
	}

	public void addRelationshipToIndex (MRelationship inRel) {
		LinkedList myTempList;
		myTempList = null;

		// Adding the relationship source info to index
		if (mRelationshipIndex.get(inRel.getSource()) == null) {
			myTempList = new LinkedList();
			mRelationshipIndex.put(inRel.getSource(), myTempList);
		}
		else if (mRelationshipIndex.get(inRel.getSource()) instanceof LinkedList) {
			myTempList = (LinkedList) mRelationshipIndex.get(inRel.getSource());
			if (myTempList.contains(inRel)) {
			// Relationship is already stored in the index
			}
			else {
				myTempList.addLast(inRel);
				mRelationshipIndex.put(inRel.getSource(), myTempList);
			}
		}

		// Adding the relationship source info to index
		if (mRelationshipIndex.get(inRel.getDestination()) == null) {
			myTempList = new LinkedList();
			mRelationshipIndex.put(inRel.getDestination(), myTempList);
		}
		else if (mRelationshipIndex.get(inRel.getDestination()) instanceof LinkedList) {
			myTempList = (LinkedList) mRelationshipIndex.get(inRel.getDestination());
			if (myTempList.contains(inRel)) {
			// Relationship is already stored in the index
			}
			else {
				myTempList.addLast(inRel);
				mRelationshipIndex.put(inRel.getDestination(), myTempList);
			}
		}
	}

	/**
	 * Creates index for Modelstorage.
	 *
	 */
	public void createIndex () {
		ListIterator it;
		MElement myTempElement;
		MRelationship myTempRelationship;
		it = nodes.listIterator();
		while (it.hasNext()) {
			myTempElement = (MElement) it.next();
			addElementToIndex(myTempElement);
		}
		it = edges.listIterator();
		while (it.hasNext()) {
			myTempRelationship = (MRelationship) it.next();
			addRelationshipToIndex(myTempRelationship);
		}
	}

	/**
	 * Returns all relationships entering an MElement.
	 *
	 */
	public LinkedList getIncomingRelationships (String inElemId) {
		LinkedList myTempList;
		MRelationship myTempRelationship;
		LinkedList out = new LinkedList();
		ListIterator it;

		if (mRelationshipIndex.get(inElemId) == null) {
			return null;
		}
		else if (mRelationshipIndex.get(inElemId) instanceof LinkedList) {
  			myTempList = (LinkedList) mRelationshipIndex.get(inElemId);
			it = myTempList.listIterator();
			while (it.hasNext()) {
				myTempRelationship = (MRelationship) it.next();
				if (  myTempRelationship.getDestination().equals(inElemId) ) {
					out.addLast(getElement(myTempRelationship.getSource()));
				}
			}
		}
		return out;
	}
}
