/**
 * @(#) MoveRelationshipsDownCommand.java
 */

package venice;

import java.util.*;


/**
 * 
 * 
 * @author	Petteri Kamppuri
 * @version	1.0
 * 
 */
public class MoveRelationshipsDownCommand extends Command
{
    /**
     * Moves dependencies for all selected nodes one level down.
     */
    public void execute()
    {
	mModel			= getAppModule().getModelStorage();
	mVisualization	= getAppModule().getGui().getVisualization();
	
	/* Get current selection from visualisation subsystem*/
	VModelElement[] mySelection = mVisualization.getSelection();
	
	if(mySelection != null) {
	    // Iterate through selected elements
	    for(int i = 0; i < mySelection.length; ++i) {
		
		VPackage myParent = mySelection[i].getVParent();
		
		LinkedList myListOfRelationships = myParent.getCompositeRelationships();
		
		if(myListOfRelationships != null) {
		    ListIterator myIterator = myListOfRelationships.listIterator(0);
		    
		    while(myIterator.hasNext()) {
			VRelationship myRel = (VRelationship) myIterator.next();
			    
			ListIterator mySubrelIterator = myRel.getSubrelationships().listIterator(0);
			
			while(mySubrelIterator.hasNext()) {
			    VRelationship mySubrelationship = (VRelationship) mySubrelIterator.next();
			    
			    if(mySubrelationship.getSource() == mySelection[i] ||
			       mySubrelationship.getDestination() == mySelection[i]) {
				
				myRel.removeSubrelationship(mySubrelationship);
				mySubrelIterator.remove();
				
				if(myRel.getSubrelationships().size() == 0) {
				    myIterator.remove();
				    OpenPackageCommand.decomposeVRelationship(myRel);
				    mVisualization.removeVRelationship(myRel);
				}
				
				mVisualization.addVRelationshipWithoutAddingOriginalReference(
						      mySubrelationship);
			    }
			}
		    }
		}
	    }
	}
    }
    
    /**
     * Private variable to hold the ModelStorage instance that is needed
     * in various places in the recursive action of this object. 
     */
    private ModelStorage  mModel		= null;
    
    /**
     * Private variable to hold the Visualization instance that is needed
     * in various places in the recursive action of this object. 
     */
    private Visualization mVisualization	= null;
    
}





