package venice;

import java.io.*;
import javax.swing.*;

/**
 * SaveCommand implements saving of .venice-files which contains both
 * the GXL model and the visualization layout.
 *
 * SaveCommand is an abstract class which can save the file to any
 * OutputStream. Subclasses of this command must implement getOutputStream()
 * method which opens output stream for writing.
 *
 * @author Hannu Laurila
 */
public abstract class SaveCommand extends Command
{
    /**
     * returns OutputStream where the data can be saved. getOutputStream()
     * may cancel saving operation by returning a null.
     */
    protected abstract OutputStream getOutputStream();

    public void execute()
    {
        Visualization myVisualization = 
            getAppModule().getGui().getVisualization();
        ModelStorage myModelStorage = 
            getAppModule().getModelStorage();  

        OutputStream myOs = getOutputStream();

        if (myOs == null)
            return;
    
        try {    
            VeniceFileHandler myVfh = new VeniceFileHandler();
            
            // write visualization to 'memory file'
            OutputStream myVisOs = myVfh.getVisualizationOutputStream();
            myVisualization.saveLayout(myVisOs);
        
            // write model to 'memory file'
            OutputStream myGXLOs = myVfh.getGXLOutputStream();
            myModelStorage.writeGXL(myGXLOs);

            // write memory file to the output stream
            myVfh.writeTo(myOs);
        }
        catch (IOException ex) {
            Util.handleException(ex);
        }
    }
}
