/**
 * @(#) ShowRequiredInterfacesCommand.java
 */

package venice;

import java.util.NoSuchElementException;
import java.util.LinkedList;
import java.util.ListIterator;

/**
 * This command show interfaces the element requires.
 * It generates new VInterface and VRealization to the interface.
 * 
 * @author	Antti Pietarinen
 * @version	1.1
 * 
 */
public class ShowRequiredInterfacesCommand extends Command
{
	/**
	 * 
	 * 
	 */
	public void execute()
	{
		mModel		= getAppModule().getModelStorage();
		mVisualization	= getAppModule().getGui().getVisualization();
		
		/* Current selection from visualisation subsystem*/
		VModelElement[] mSelection = mVisualization.getSelection();
		MRelationship mRelationship  = null;

		LinkedList myList;
		ListIterator myIterator;
                LinkedList mySubsList;
                ListIterator mySubsIterator;

		
		VInterface mReqInterface     = null;
		MElement mOrgInterface     = null;
		VRealization mNewRealization = null;
		

		for(int i = 0 ; i < mSelection.length; i++){ // for each selectedNode
		    myList = mModel.getDependencies(mSelection[i].getId());
		    myIterator = myList.listIterator(0);
		    
		    if(mSelection[i] instanceof ZPackage){
			if(!((ZPackage)mSelection[i]).getRequiredRealizations().isEmpty())
			    break;
		    }		    
		    else{
			if(!((ZVisualLeafModelElement)mSelection[i]).getRequiredRealizations().isEmpty())
                            break;
		    }

		    while( myIterator.hasNext() ){ // for each Relationship
			try{
			    mRelationship = (MRelationship) myIterator.next();
			}catch(NoSuchElementException e){
			    break;
			}
			/*if relationship to interface*/
			if( mRelationship.getSource().equals( mSelection[i].getId() ) &&
			    mRelationship instanceof MDependency &&
			   (mOrgInterface = mModel.getElement( mRelationship.getDestination())) instanceof MInterface ){
				/*new VInterface*/
				mReqInterface = mVisualization.newVInterface( mRelationship.getDestination() +"(RI)"+ mSelection[i].getId() );
				mReqInterface.setName(mOrgInterface.getName());
				mReqInterface.setInterfaceType(2); // required
				mReqInterface.setViewMode(1); // circle
				mReqInterface.setOperations( ( (VInterface)mVisualization.findElementById(mOrgInterface.getId()) ).getOperations());
				/*new VRealization*/
				mNewRealization = mVisualization.newVRealization( mRelationship.getDestination() +"(RI)"+ mSelection[i].getId()+":REA" ); // fixme ID
				mNewRealization.setDestination(mReqInterface);
				mNewRealization.setSource(mSelection[i]);

				if(mSelection[i] instanceof ZPackage)
                                    ((ZPackage)mSelection[i]).addToRequiredRealizations(mNewRealization);
                                else
                                    ((ZVisualLeafModelElement)mSelection[i]).addToRequiredRealizations(mNewRealization);

				if( mModel.getParent(mOrgInterface.getId()) != null ){
				    ( (VPackage)mVisualization.findElementById(mModel.getParent( mOrgInterface.getId()).getId() )).addVChild(mReqInterface);
				}
				else{
				    mVisualization.getRootPackage().addVChild(mReqInterface);
				}
				
				mVisualization.addVRelationshipWithoutAddingOriginalReference(mNewRealization);
				
			}
		    }
		}
	}
    
	/**
	 * Private variable to hold the ModelStorage instance that is needed
	 * in various places in the recursive action of this object. 
	 */
	private ModelStorage  mModel		= null;
	
	/**
	 * Private variable to hold the Visualization instance that is needed
	 * in various places in the recursive action of this object. 
	 */
	private Visualization mVisualization	= null;

}
